IF OBJECTPROPERTY(object_id('Sales'), 'IsUserTable') = 1
    DROP TABLE Sales
go
CREATE TABLE Sales
(
    InvNum         int        NOT NULL    identity,
    InvDate        datetime   NOT NULL    CONSTRAINT SalesDateDflt DEFAULT getdate(),
    CustomerID     int        NULL        -- Foreign Key... and so forth
)
go
DECLARE @CustCounter    int
SELECT @CustCounter = 1
WHILE 1=1
BEGIN
    INSERT Sales (CustomerID) VALUES (@CustCounter)
    IF @CustCounter < 5
        SELECT @CustCounter = @CustCounter + 1
    ELSE
        SELECT @CustCounter = 1
END
go
SELECT * FROM Sales
SELECT count(*) FROM Sales
go
DELETE Sales 
    WHERE invnum = 1234
-- Never give direct DELETE permissions to the base tables
go
-- Can prevent this with a VERY expensive (albeit automatic)
-- trigger
go
CREATE TRIGGER SalesDeleteTrigger
ON Sales
FOR DELETE
AS
DECLARE @RowCount    int
SELECT @RowCount = @@rowcount

IF @RowCount > 1
BEGIN
    RAISERROR ('You have attemped to delete %i rows. The sales table does not allow multirow operations. Please retry DELETE on a row by row basis.', 16, 1, @rowcount) WITH LOG
    ROLLBACK TRANSACTION
    RETURN
END

-- OR you can arbitrarily set a MAX batch delete size?
-- IF @RowCount => 100
-- BEGIN
--     RAISERROR ('You have attemped to delete %i rows. The maximum number of rows that a single DELETE can support against this table is 100 rows. Please retry DELETE on a smaller set.', 16, 1, @rowcount) WITH LOG
--     ROLLBACK TRANSACTION
--     RETURN
-- END
go
-- A MUCH better choice would be to eliminate this from occuring in the
-- first place!!!
go
CREATE PROCEDURE DeleteASales
	@invnum	int
AS
DELETE sales
	WHERE invnum = @invnum
-- While overly simplified (ie. no error handling, etc.) this 
-- still shows how a single user cannot delete more than one row
-- at a time.
go
EXEC DeleteASales 1234
go

-- Creative (read: malicious) users are harder to prevent 
--      -> Application Roles might be an option
DECLARE @counter	int
SELECT @counter = MIN(invnum) FROM sales
WHILE @counter < SELECT MAX(invnum) FROM sales
BEGIN
	EXEC deleteasales @counter
	SELECT @counter = MIN(invnum) FROM sales
			WHERE invnum > @counter
END
go
